mod commands;
mod util;
mod handler;

use std::error::Error;

use lavalink_rs::client::LavalinkClient;
use lavalink_rs::model::events::Events;
use lavalink_rs::model::UserId;
use lavalink_rs::node::NodeBuilder;
use serenity::all::OnlineStatus;
use serenity::async_trait;
use serenity::builder::{CreateEmbed, CreateEmbedAuthor, CreateEmbedFooter, CreateInteractionResponse, CreateInteractionResponseMessage};
use serenity::gateway::ActivityData;
use serenity::model::application::{Command, Interaction};
use serenity::model::gateway::Ready;
use serenity::prelude::*;
use util::config;

use crate::handler::lavalink_event_handler;


struct Handler;

pub struct Data {
    pub lavalink: LavalinkClient,
}

#[async_trait]
impl EventHandler for Handler {
    async fn interaction_create(&self, ctx: Context, interaction: Interaction) {
        if let Interaction::Command(command) = interaction {
            let content = match command.data.name.as_str() {
                "info" => Some(commands::info::run(&ctx, &command, &command.data.options())),
                "play" => Some(commands::play::run(&ctx, &command, &command.data.options())),
                "stop" => Some(commands::stop::run(&ctx, &command, &command.data.options())),
                _ => Some(CreateEmbed::new()
                .author(CreateEmbedAuthor::new("Rustendo"))
                .description("Cannot find command")
                .footer(CreateEmbedFooter::new("Error during execution"))),
            };

            if let Some(embed) = content {
                let data = CreateInteractionResponseMessage::new().embed(embed);
                let builder = CreateInteractionResponse::Message(data);
                if let Err(why) = command.create_response(&ctx.http, builder).await {
                    println!("Cannot respond to slash command: {why}");
                }
            }
        }
    }

    async fn ready(&self, ctx: Context, ready: Ready) {
        println!("{} is connected!", ready.user.name);

        let _command = Command::create_global_command(&ctx.http, commands::info::register()).await;
        let _command = Command::create_global_command(&ctx.http, commands::stop::register()).await;
        let _command = Command::create_global_command(&ctx.http, commands::play::register()).await;

        println!("Created all public / commands");
    }
}

#[tokio::main]
async fn main() {
    println!(r"__________               __                      .___      
\______   \__ __ _______/  |_   ____    ____   __| _/____  
 |       _/  |  |  ___/\   __\_/ __ \  /    \ / __ |/  _ \ 
 |    |   \  |  |___ \  |  |  \  ___/ |   |  | /_/ (  <_> )
 |____|_  /____/____  > |__|   \___  >|___|  |____ |\____/ 
        \/          \/             \/      \/     \/       
");


    // Load config
    let config = config::load().unwrap();
    // Set status
    let status = OnlineStatus::DoNotDisturb;
    let activity = ActivityData::streaming("music", "https://twitch.tv/moonleaytv").unwrap();

    let node_builder = NodeBuilder {
    hostname: config.lavalink_address,
    password: config.lavalink_password,
    user_id: UserId(config.user_id),
    ..Default::default()
    };

    // let event = Events::default();
    let event = Events {
        ready: Some(lavalink_event_handler::ready_event),
        ..Default::default()
    };

    // Connect to lavalink
    let lavalink_client = LavalinkClient::new(event, vec![node_builder]);
    tokio::spawn(async move {
        lavalink_client.start().await;
    });

    // Build the client
    let mut client = Client::builder(config.discord_token, GatewayIntents::empty())
        .event_handler(Handler)
        .status(status)
        .activity(activity)
        .await
        .expect("Error creating client");

    // Finally, start a single shard, and start listening to events.
    //
    // Shards will automatically attempt to reconnect, and will perform exponential backoff until
    // it reconnects.
    if let Err(why) = client.start().await {
        println!("Client error: {why:?}");
    }
}
